
/* ************************************************************************
*   FILE: TWI Salve1    
*
*   DESC: Atmega8 TWI hardware slave.
*                            
* AUTHOR: Zoltan Gradwohl 
*   DATA: 05/29/05
*  EMAIL: zgrad@web.de
************************************************************************ */



#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <inttypes.h>



// um Register auch bitweise in Binrformat ansprechen zu knnen
#define BIN8(b7,b6,b5,b4,b3,b2,b1,b0) ((unsigned char)\
                          (b0 << 0)\
                        | (b1 << 1)\
                        | (b2 << 2)\
                        | (b3 << 3)\
                        | (b4 << 4)\
                        | (b5 << 5)\
                        | (b6 << 6)\
                        | (b7 << 7))

#define SLAVE_ADR  0x04					// define own slave adress

void uart_send_u8(uint8_t y);



volatile unsigned char twi_val1;

int main (void) 
{

	PORTB=BIN8(1,1,1,1,0,1,1,0); 		// pull up PINs... (nur wenn in DDR als Eingang def)
	PORTC=BIN8(1,0,0,1,1,1,1,1); 		// pull up PINs... (nur wenn in DDR als Eingang def)
	PORTD=BIN8(0,0,1,1,1,0,1,1); 		// pull up PINs... (nur wenn in DDR als Eingang def) 
	
	DDRB=  BIN8(0,0,0,0,1,0,0,1);		// Datenrichtung PortB 	0=Eingang 1=Ausgang
	DDRC=  BIN8(0,0,0,0,0,0,0,0);		// Datenrichtung PortC
	DDRD=  BIN8(1,1,0,0,0,1,0,0);		// Datenrichtung PortD
	
	UBRRL=12;										// Set baudrate 38400  (8MHz/ 16*(UBRR+1)=baud  =12	
	UCSRB|= _BV(RXEN)|_BV(TXEN)|_BV(RXCIE);		// UART recieve on, UART Send on, UART RX complete interrupt enable	
	
	
	
	// TWI-Schnittstelle initialisieren  Slave Transmitter Mode
	TWAR=0;
	TWAR|= (SLAVE_ADR<<1); 	// Receiver adress
	TWCR = (1<<TWEN)|(1<<TWEA)|(1<<TWIE)|(1<<TWINT);  
	
	// TWEN : TWI Enable
	// TWSTA : 1=master 0=slave
	// TWEA : TWI Enable Acknowledge Bit
	// TWIE : TWI Interrupt Enable
	
	uart_send_u8(0x0F);	
	sei();

	for (;;){asm volatile ("nop");}
	
	return 0;
} 





SIGNAL(SIG_2WIRE_SERIAL) 
{  
	switch (TWSR)
	{
		//****************************************
		// Slave status (receiver mode)
		
		case(0x60):{ // Own SLA+W has been received; ACK has been returned
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);	// In next action: Data byte will be received and ACK will be returned
			break;
		} 
	
		case(0x68):{   //Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);
			break;
		} 
		
		case(0x80):{ // Previously addressed with own SLA+W; data has been received; ACK has been returned
			twi_val1=TWDR;
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);
			break;
		}
		
		case(0xA0):{   //A STOP condition or repeated START condition has been received while still addressed as Slave
			//i2c_val2=TWDR;  // nicht zweimal lesen!!
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);
			uart_send_u8(twi_val1);								// send data to PC
			break;
		} 

		
		


			//****************************************
		// Slave status (transmitter mode)
 
		case(0xA8):{ //Own SLA+R has been received; ACK has been returned 
			TWDR =twi_val1; 
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);
			break;	
		}
		case(0xC0):{  // Data byte in TWDR has been transmitted; NOT ACK has been received
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);
			break;	
		}
		case(0xC8):{  // Data byte in TWDR has been transmitted;  ACK has been received
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);
			break;	
		}
		case(0xB8):{  // Data byte in TWDR has been transmitted; ACK has been received 
		     TWDR =twi_val1;  
			TWCR=(1<<TWEN)|(1<<TWINT)|(1<<TWEA)|(1<<TWIE);
			break;	
		}
	}	
}



//////////////////////////////////////////////////////////////////////////////////////////
// USART sendet 8 bit										
////////////////////////////////////////////////////////////////////////////////////////// 
void uart_send_u8(uint8_t data)
{	
	while(!(UCSRA & (1<<UDRE)));		// Warten, bis UDRE eins ist-->UDR ist leer	
	UDR=(uint8_t)data;		  			// data_to_send in das UDR Register schreiben
}





