/* ************************************************************************
* 	Original source by Dave Watson --- djwatson@u.washington.edu
* 	Created 6/10/2000; 11:00:44 AM
*
*
*   DESC: I2C data transfer between AVR(or any I2C device) and TI92+.
*
*
* 	Modifications by Zoltan Gradwohl --- zgrad@web.de
* 	28-May-2005
************************************************************************ */




// Compile without optimisation because of delay() if defined !!!

// #define USE_GRAY_PATCH
#define USE_LONGMUL_PATCH
#define USE_LONGDIV_PATCH
#define USE_INITS_PATCH

#define ESC_ROW 0xffbf
#define ESC_KEY 1
         
//#define delay

//#define SAVE_SCREEN

#include <nostub.h>
#include <tigcclib.h>

void I2C_Init(void);
void I2C_Start(void);
void I2C_Stop(void);
void I2C_SDA(char state);
void I2C_SCK(char state);
void I2C_Clock(void);
int  I2C_ReadSDA(void);
char I2C_Poll(char address);
void I2C_PutByte(char buf);
char I2C_GetByte(void);
void I2C_GiveACK(void);
void I2C_GiveNOTACK(void);
int  I2C_GetACK(void);
int  I2C_Read(char address,unsigned int Number_of_bytes, char* data);
int  I2C_ReadRep(char address,unsigned int Number_of_bytes, char* data);
int  I2C_Write(char address,unsigned char Number_of_bytes, char* data);
int  I2C_WriteRep(char address,unsigned char Number_of_bytes, char* data);
void reset(void);
void delay(void);

void Draw_Statusbar(unsigned long memused,unsigned long totalmem);
char Send_Receive_AVR(char *datatosend);


static char *const point=(char *)0x60000C;
static char *const point2=(char *)0x60000E;

unsigned int slaveaddress=0x04,pages=512;
unsigned char pagesize=128;




///////////////////////////////////////////////////////////
// MAIN ///////////////////////////////////////////////////
///////////////////////////////////////////////////////////

void _main(void)
{
	char state=1;
	int key=0;
	
	unsigned long memoryused=65538,totalmemory;
	char selecdata[10];
	
	*point=0x40;
	
	totalmemory=(long)pages*(long)pagesize;
	
	
	
	do
	{		
		if(key==KEY_F1){
			if(slaveaddress<127) 
				slaveaddress++;
		}
		if(key==KEY_F2){
			if(slaveaddress>2) 
				slaveaddress--;
		}
		
		ClrScr();
		Draw_Statusbar(memoryused,totalmemory);
			
			
		if(key==KEY_LEFT){
    	selecdata[0]=1;
    	state = Send_Receive_AVR(selecdata); 
    } 
    else if(key==KEY_RIGHT){
    	selecdata[0]=2;
    	state = Send_Receive_AVR(selecdata); 
    }  
   	else if(key==KEY_UP){
    	selecdata[0]=3;
    	state = Send_Receive_AVR(selecdata); 
    }  
  	else if(key==KEY_DOWN){
    	selecdata[0]=4;
    	state = Send_Receive_AVR(selecdata); 
    }  
  
  
	
		if(state==0)
			DlgMessage("Error!", "Device not responding!   Please check all connections and settings.",BT_NONE, BT_OK);	
	  state=1;
	  
	  
		Draw_Statusbar(memoryused,totalmemory);
		GKeyFlush();
		key=ngetchx();
	}
	while(key!=KEY_ESC);
	
	reset();
	return;
}





char Send_Receive_AVR(char *datatosend)
{
		char state=0;
		char datareceived[2]={0, 0};
 			
	  state = I2C_WriteRep((slaveaddress<<1), 1, datatosend);
	  
	  int i;	
		for(i=0;i<100;i++) asm volatile ("nop");		// pause	
	  
	  state = I2C_ReadRep((slaveaddress<<1), 2, datareceived);
	  
	  printf_xy(0,6,"Data Transmitted: %u" , datatosend[0]&0xFF);
	  printf_xy(0,13,"Data Received: %u %u",  datareceived[0]&0xFF, datareceived[1]&0xFF);
	  
	  return(state);
}





void Draw_Statusbar(unsigned long memused,unsigned long totalmem)
{
	SCR_RECT infobarrect = {0,93,229,100};
	SCR_RECT fullscreen  = {0,0,230,100};
	char tmpstr[80];
	
	FontSetSys(F_4x6);

  ScrRectFill(&infobarrect,&fullscreen,A_NORMAL);
  sprintf(tmpstr, "Device Address (F1-F2): 0x%x                                  Mem:%ld/%ld",slaveaddress,memused,totalmem);
  DrawStrXY(1,87,tmpstr,A_NORMAL);
  DrawStrXY(2,94," KeyLeft ",A_REVERSE);
  DrawStrXY(37,94," KeyRight ",A_REVERSE);
  DrawStrXY(75,94," KeyUp ",A_REVERSE);
  DrawStrXY(102,94," KeyDown ",A_REVERSE);
  DrawStrXY(210,94," ESC ",A_REVERSE);
}







///////////////////////////////////////////////////////////
// I2C Routines ///////////////////////////////////////////
///////////////////////////////////////////////////////////
// Original source by Dave Watson --- djwatson@u.washington.edu
// Created 6/10/2000; 11:00:44 AM

void reset(void){
	*point2=0x50;
	*point=0x8D;
}

void delay(void){
	int i, d=1;
	for(i=2; i>0; --i) d*=2;  //i=5
}

int I2C_ReadSDA(void){
	return !((*point2&0x04)>>2);
}

void I2C_SDA(char state){
	if(state==1)
		*point2&=0xFE;
	else
		*point2|=0x01;
#ifdef delay
	delay();
#endif
}

void I2C_SCK(char state){
	if(state==1)
		*point2&=0xFD;
	else
		*point2|=0x02;
#ifdef delay
	delay();
#endif
}

void I2C_Clock(void){
	I2C_SCK(1);
	I2C_SCK(0);
}

// it seems that this is not needed
void I2C_Init(void){ // call this immediately after power-on
	int n;
	I2C_SDA(1);
	I2C_SCK(0);
	for(n=0; n<4; ++n){
		I2C_Stop();
	}
}

void I2C_Start(void){
	I2C_SDA(1);
	I2C_SCK(1);
	I2C_SDA(0); 
	I2C_SCK(0); 
}

void I2C_Stop(void){
	I2C_SDA(0); 
	I2C_SCK(1); 
	I2C_SDA(1); 
}

void I2C_PutByte(char buf){
	int n;
	for (n=7; n>=0; --n){
		I2C_SDA((buf>>n)&0x01);
		I2C_Clock(); 
	}
	I2C_SDA(1); 
}

char I2C_GetByte(void){
	char buf=0;
	int n;
	for (n=7; n>=0;--n){
		I2C_SCK(1);
		buf |= I2C_ReadSDA()<<n;
		I2C_SCK(0);
	}
	I2C_SDA(1);
	return buf;
}

void I2C_GiveACK(void){
	I2C_SDA(0);
	I2C_Clock();
	I2C_SDA(1);
}

void I2C_GiveNOTACK(void){
	I2C_SDA(1);
	I2C_Clock();
}

int I2C_GetACK(void){
	int i;
	I2C_SDA(1);
	I2C_SCK(1);
	// don't wait for ACK, but give slow components some time
	// to respond
	// however, I don't think that this is needed, the slave should
	// respond immediately after the 8th bit
	for(i=10; i>0; --i){
		if(I2C_ReadSDA()==0){
			I2C_SCK(0);
			return(1);
		}
	}
	I2C_SCK(0);
	return 0;
}

char I2C_Poll(char address)
{
	unsigned int x=0;
	char retval=0;
//acknowledge polling
	do
	{
		I2C_Start();
		I2C_PutByte(address&0xfe);
		x++;
	}
	while(!(retval=I2C_GetACK()) && x<1000);
	I2C_Stop();
	return retval;
}

/* this concludes the low-level set of instructions for the I2C driver
The next functions will handle the telegram formatting on a higher level */

// a complete read sequence with Stop
int I2C_Read(char address,unsigned int Number_of_bytes, char* data){
	int state;
	state=I2C_ReadRep(address, Number_of_bytes, data);
	I2C_Stop();
	return state;
}

// a read sequence without Stop, allows for Repeated Start
int I2C_ReadRep(char address, unsigned int Number_of_bytes, char* data){
	unsigned int x;

	I2C_Start();
	I2C_PutByte(address|0x01);
	if(!I2C_GetACK()){
		return 0;
	}
	for(x=0; x<Number_of_bytes; x++){
		data[x]=I2C_GetByte();
		if (x<Number_of_bytes-1)
			I2C_GiveACK();
		else
			I2C_GiveNOTACK();
	}
	return 1;
}

// a complete write sequence with Stop
int I2C_Write(char address, unsigned char Number_of_bytes, char* data){
	int state;
	state=I2C_WriteRep(address, Number_of_bytes, data);
	I2C_Stop();
	return state;
}

// a write sequence without Stop, allows for Repeated Start
int I2C_WriteRep(char address, unsigned char Number_of_bytes, char* data){
	int x,i=0;
retryw:
	i++;
	if (i>10) return(0);
	I2C_Start();
	I2C_PutByte(address&0xfe);
	if(!I2C_GetACK())
		goto retryw;
	for(x=0; x<Number_of_bytes; ++x){
		I2C_PutByte(data[x]);
		if(!I2C_GetACK())
			goto retryw;
	}
	return 1;
}
